
import os
import io

import streamlit as st
import seaborn as sns
import matplotlib
import time

from defermi import DefectsAnalysis
from defermi_gui.defaults import set_defaults
from defermi_gui.initialize import initialize_state_variables, initialize_defects_analysis, initialize_complete_dataframe
from defermi_gui.inputs import upload_file, band_gap_vbm_inputs, load_file#, filter_entries
from defermi_gui.chempots import chempots
from defermi_gui.dos import dos
from defermi_gui.thermodynamics import thermodynamics
from defermi_gui.utils import init_state_variable, save_session




pages_dict = {
    'home': st.Page('home.py',title='Home',icon=':material/home:'),
    'overview': st.Page('overview.py',title='Overview',icon=':material/view_module:'),
    'data': st.Page('data.py',title='Data',icon=':material/table:'), 
    'formation_energies': st.Page('formation_energies.py',title='Formation Energies',icon=':material/line_axis:'),  
    'doping': st.Page('doping.py',title='Doping',icon=':material/stacked_line_chart:'),
    'brouwer': st.Page('brouwer.py',title='Brouwer',icon=':material/ssid_chart:'),
    'fermi_level': st.Page('fermi_level.py',title='Fermi Level',icon=':material/show_chart:'),
    'ctls': st.Page('ctls.py',title='Charge Transition Levels',icon=':material/insert_chart:'),
    'binding_energies': st.Page('binding_energies.py',title='Binding Energies',icon=':material/stacked_line_chart:')     
}


with st.sidebar:
    file = upload_file()
    load_file(file)

    initialize_state_variables()
    initialize_complete_dataframe()
    initialize_defects_analysis(st.session_state['complete_dataframe'])

    band_gap_vbm_inputs()
    st.divider()

    chempots()

    dos()
    thermodynamics()
    
set_defaults()

# exclude binding energies if there are no complexes
pages = []
for k,v in pages_dict.items():
    if k == 'binding_energies':
        if st.session_state.da:
            if 'DefectComplex' in st.session_state.da.types:
                pages.append(v)
    else:
        pages.append(v)

page = st.navigation(pages,expanded=True)

filename = st.session_state['session_name'] + '.defermi'
save_session(filename)
page.run()







import tempfile
import os
import time
import json
import io

import matplotlib
import streamlit as st
import pandas as pd



from defermi import DefectsAnalysis 
from defermi_gui.info import file_loader_info, band_gap_info
from defermi_gui.utils import load_session, init_state_variable, reset_session


def reset_and_update_data():
    st.session_state.clear()
    st.session_state['uploader_changed'] = True
    return

def upload_file():
    st.markdown('## 📂 File')
    init_state_variable('uploader_changed',value=False)
    cols = st.columns([0.9,0.1])
    with cols[0]:
        uploaded_file = st.file_uploader("upload", type=["csv","pkl","defermi"], on_change=reset_and_update_data, label_visibility="collapsed")
    with cols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(file_loader_info)
    return uploaded_file


def load_dataframe(uploaded_file):
    name = uploaded_file.name
    if name.endswith('.csv'):
        dataframe = pd.read_csv(uploaded_file) # Streamlit's UploadedFile can be read directly for csv
    elif name.endswith('.pkl'):
        dataframe = pd.read_pickle(io.BytesIO(uploaded_file.getvalue())) # pass the bytes for pickles
    else:
        st.error('Dataset format must be "csv" or "pkl"')
        return None
    return dataframe


def load_file(uploaded_file):
    init_state_variable('session_loaded',value=False)
    if uploaded_file:
        if ".defermi" in uploaded_file.name and not st.session_state['session_loaded']:
            load_session(uploaded_file) 
            st.session_state['session_loaded'] = True
        elif '.defermi' not in uploaded_file.name and not st.session_state['session_loaded']:
            df = load_dataframe(uploaded_file)            
            df['Include'] = [True for i in range(len(df))]
            cols = ['Include'] + [col for col in df.columns if col != 'Include']
            df = df[cols]
            st.session_state['input_dataframe'] = df
            st.session_state['session_name'] = uploaded_file.name.split('.')[0]
    return


def band_gap_vbm_inputs():
    init_state_variable('band_gap',value=None)
    init_state_variable('vbm',value=0.0)
    if not st.session_state['complete_dataframe'].empty:
        cols = st.columns([0.45,0.45,0.1])
        with cols[0]:
            value = st.session_state['band_gap']            
            band_gap = st.number_input("Band gap (eV)", value=value, step=0.1, placeholder="Enter band gap", key='widget_band_gap')
            st.session_state['band_gap'] = band_gap
            if st.session_state['band_gap'] is None:
                st.warning('Enter band gap to begin session')
            
        with cols[1]:
            value = st.session_state['vbm']
            vbm = st.number_input("VBM (eV)", value=value, step=0.1, key='widget_vbm')
            st.session_state['vbm'] = vbm
        with cols[2]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(band_gap_info)
    return 







import streamlit as st

from pymatgen.core.composition import Composition

from defermi.chempots.generator import generate_elemental_chempots, generate_chempots_from_condition
from defermi_gui.info import chempots_info
from defermi_gui.utils import init_state_variable, widget_with_updating_state, reset_home_figures

def chempots():
    """
    GUI elements for chemical potentials 
    """
    if st.session_state.da:
        da = st.session_state.da
        init_state_variable('chempots',value={})
        chempots_DB = {}

        def set_chempots_from_DB(composition,condition):
            if composition:
                chempots_DB = pull_chempots_from_condition(
                                                        composition=composition,
                                                        condition=condition)
                elements_in_composition = [el.symbol for el in composition.elements]
                elements_to_pull = [el for el in da.elements if el not in elements_in_composition]
                chempots_elemental = pull_elemental_chempots_from_MP(elements_to_pull)
                for el,mu in chempots_elemental.items():
                    chempots_DB[el] = mu
            else:
                chempots_DB = pull_elemental_chempots_from_MP(da.elements)
            
            for el,mu in chempots_DB.items():
                st.session_state[f'widget_chempot{el}'] = mu
            reset_home_figures()
            return
        

        cols = st.columns([0.36,0.55,0.09])
        with cols[0]:
            st.markdown("**Chemical Potentials (eV)**")
        with cols[1]:
            with st.expander('🗄️ Materials Project Database'):
                help = ""
                composition = st.text_input('Reference composition',
                                                    value='',
                                                    placeholder='Empty for elemental chemical potentials',
                                                    help = 'Composition of the pristine material',
                                                    key='widget_composition_DB')
                if composition:
                    composition = Composition(composition)
                    subcols = st.columns(2)
                    with subcols[0]:
                        options = [el.symbol for el in composition.elements]
                        if 'O' in options:
                            index = options.index('O')
                        element = st.radio(label='Element',options=options,index=index)
                    with subcols[1]:
                        condition_str = st.radio(label='Condition',options=['middle','rich','poor'],index=0)
                    condition = '-'.join([element,condition_str])
                else:
                    condition = None
                st.button('Pull',on_click=set_chempots_from_DB,args=[composition,condition])
                    
        with cols[2]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(chempots_info)

        mu_string = "μ"
        
        ncolumns = 4
        cols = st.columns(ncolumns)
        for idx,el in enumerate(da.elements):
            col_idx = idx%ncolumns
            with cols[col_idx]:
                if el in st.session_state['chempots']:
                    widget_key = f'widget_chempot{el}'
                    if widget_key not in st.session_state:
                        st.session_state[widget_key] = st.session_state['chempots'][el] or 0.0

                mu = st.number_input(f"{mu_string}({el})", max_value=0.0,step=0.5, key=f'widget_chempot{el}',on_change=reset_home_figures)
                st.session_state.chempots[el] = mu

        st.divider()



def pull_elemental_chempots_from_MP(elements,thermo_type='GGA_GGA+U',**kwargs):
    """
    Generate chemical potentials for reference elemental phases from the
    Materials Project database.

    Parameters
    ----------
    elements : list
        List of strings with element symbols.
    thermo_type : str
        The thermo type to pass to MP database. 
    kwargs : dict
        Kwargs to pass to `get_phase_diagram_from_chemsys`.

    Returns
    -------
    Chempots object
    """
    import base64
    from defermi.tools.materials_project import MPDatabase

    API_KEY = base64.b64decode('Q0FVMk8yODZmRUI2cGJWOUszTU9qblFFUFJkZW9BQXg=').decode()
    chempots = generate_elemental_chempots(
                                        elements,
                                        API_KEY=API_KEY,
                                        thermo_type=thermo_type,
                                        **kwargs)
    return chempots


def pull_chempots_from_condition(composition,condition,thermo_type='GGA_GGA+U',**kwargs):

    import base64
    from defermi.tools.materials_project import MPDatabase

    API_KEY = base64.b64decode('Q0FVMk8yODZmRUI2cGJWOUszTU9qblFFUFJkZW9BQXg=').decode()
    chempots = generate_chempots_from_condition(
                                            composition=composition,
                                            condition=condition,
                                            API_KEY=API_KEY,
                                            thermo_type=thermo_type,
                                            **kwargs)
    return chempots






import os
import defermi_gui

def run_gui():
    path_gui_main = defermi_gui.__file__.replace('__init__.py', 'main.py')
    os.system(f"streamlit run {path_gui_main}")

def main():
    run_gui()

if __name__ == "__main__":
    main()



presets_info = """
Presets do NOT contain real data! All energies are made-up numbers.
Go to the **Data** section and experiment with numbers. See how changing parameters affects the results.
"""

dataframe_info = """
- `name` : Name of the defect, naming conventions described below.
- `charge` : Defect charge.
- `multiplicity` : Multiplicity in the unit cell.
- `energy_diff` : Energy of the defective cell minus the energy of the pristine cell in eV.
- `bulk_volume` : Pristine cell volume in $\\mathrm{\\AA^3}$

Additionally, you can include correction terms by adding columns named
`corr_{insert corr name}` (eg `corr_elastic`). Each value in columns with this name will be added to the formation energy. 

Defect naming: (element = $A$)
- Vacancy: `'Vac_A'` (symbol=$V_{A}$)
- Interstitial: `'Int_A'` (symbol=$A_{i}$)
- Substitution: `'Sub_B_on_A'` (symbol=$B_{A}$)
- Polaron: `'Pol_A'` (symbol=${A}_{A}$)
- DefectComplex: `'Vac_A;Int_A'` (symbol=$V_A - A_i$)
"""

file_loader_info = f"""
Load session file (`.defermi`) or dataset file (`.csv`,`.pkl` or `.json`)  

`defermi`:Restore previous saved session\n
`json`: Exported `DefectsAnalysis` object from the `python` library, not generated manually\n
`csv` or `pkl`: Rows are defect entries, columns are:
{dataframe_info}
"""

band_gap_info = """
Band gap and valence band maximum of the pristine material in eV. 
"""

dataset_info = f"""
Dataset containing defect entries (`pandas.DataFrame`).\n
Toggle **Include** to add or remove the defect entry from the calculations.\n
Rows are defect entries, columns are:\n
{dataframe_info}\n

Options:
- **Reset**: restore the original dataset.
- **Save csv**: Save customized dataset as `csv` file.
- **Add column**: Add column to dataset.
"""

df_include_info = """
Check the box to include the entry in the analysis 
"""

df_name_info = """
Name of the defect entry.
Naming rules: (element = $A$)
- Vacancy: `'Vac_A'` (symbol=$V_{A}$)
- Interstitial: `'Int_A'` (symbol=$A_{i}$)
- Substitution: `'Sub_B_on_A'` (symbol=$B_{A}$)
- Polaron: `'Pol_A'` (symbol=${A}_{A}$)
- DefectComplex: `'Vac_A;Int_A'` (symbol=$V_A - A_i$)
"""

df_charge_info = """
Charge of the defect (`int` or `float`)
"""

df_multiplicity_info = """
Multiplicity of the defect in the unit cell (`int`)
"""

df_energy_diff_info = """
Energy of the defective cell minus the energy of the pristine cell in eV
"""

df_bulk_volume_info = """
Volume of the unit cell in $\AA$
"""


chempots_info = """
Chemical potential of the elements that are exchanged with a reservoirs when defects are formed.\n

Formation energies depend on the chemical potentials as:\n
$$ \\Delta E_f = E_D - E_B + q(\\epsilon_{VBM} + \\epsilon_F) - \\color{blue} \\sum_i \\Delta n_i \\mu_i $$ \n

where $\\Delta n_i$ is the number of particles in the defective cell minus the number in the pristine cell for species $i$.\n

Chemical potentials can also be pulled from the Materials Project database, click **Materials Project Database**
to open the window. If **Reference composition** is left empty, chemical potentials relative to the elemental phases 
are pulled. If a compostition is specified, the phase diagram relative to the components in the target phase is retrieved,
and a dialog will appear to select which element and which condition should be used as reference.
"""

dos_info = """
Parameters for the calculation of electrons and holes concentration. Possible choices are:\n
$\\mathbf{m*/m_e}$: Effective masses of electrons (e) and holes relative to the electron mass.\n
**DOS**: Computed density of states of the pristine material. Format is either a dictionary:
- `'energies'` : list or np.array with energy values
- `'densities'` : list or np.array with total density values
- `'structure'` : pymatgen `Structure` of the material, needed for DOS volume and charge normalization

-- Alternatively, a pymatgen `Dos` object (`Dos`, `CompleteDos`, or `FermiDos`) exported as `json`.
Click on **Database** and enter the desired composition to pull the `CompleteDos` object from the 
Materials Project Database.
"""

quenching_info = """
Run simulations in quenching conditions.\n
Defect concentrations are computed in charge neutral conditions at the input **Temperature(K)**,
but charges are equilibrated at **Quench Temperature (K)**. This simulates conditions where defect mobility is 
low and the high-temperature defect distribution is frozen in at low temperature.

**Quenching mode** options:
- **species**: Fix concentrations of defect species (identified by `name`).
- **elements**: Fix concentrations of elements, concentrations of individual 
                species containing the quenched elements are computed according 
                to the relative formation energies. Vacancies are considered 
                separate elements.

Select which species or elements to quench with **Select quenched species**. Defects not in the quenching list
are equilibrated at **Quench Temperature**.
"""

external_defects_info = """
Extrinsic defects contributing to charge neutrality that are NOT present in defect entries. 
They are considered in the Brouwer diagram and doping diagram calculations. \n
There is no requirement for the defect name, if a name fits one of the naming conventions,
the corrisponding symbol will be printed.
"""

oxygen_ref_info = """
$\\mu_O(0K,p^0)$ is the chemical potential of oxygen at $T = 0 K$ and standard pressure $p^0$.\n

The oxygen partial pressure for the Brouwer diagrams is connected to the chemical potential of oxygen as:\n
$$\\mu_O(T,p_{O_2}) = \\mu_O(T,p^0) + (1/2) k_B T \\; \\mathrm{ln} (p_{O_2} / p^0) $$

where:
$$\\mu_O(T,p^0) = \\mu_O (0 K,p^0) + \\Delta \\mu_O (T,p^0) $$

The value of $\\mu_O$ in the **Chemical Potentials** section is ignored for the calculation of the Brouwer diagram.
"""

precursors_info = """
Conditions for the definition of the chemical potentials as a function of the oxygen partial pressure.
They represent the reservoirs that are in contact with the target material.\n

Each entry requires the composition and the energy per formula unit (p.f.u) in eV. Click on **Database** to pull
the energy for that composition from the Materials Project Database.\n 
Starting from the chemical potential of oxygen, the other chemical potentials are determined by the constraints 
$ E_{\\mathrm{pfu}} = \\sum_s c_s \\mu_s $, where $c_s$ are the stochiometric coefficients and $\\mu_s$ the chemical potentials.

For oxides with maximum 2 components, the target material itself is enough to determine the chemical potential of the other species.
For target oxides with more that 2 components, at least 2 compositions are needed to determine all chemical potentials.
Often these phases are chosen to be the precursors in the synthesis of the target material.\n

All elements that are not present in the entries compositions are excluded from the Brouwer diagram calculations.\n
The values in the **Chemical Potentials** section are ignored for the calculation of the Brouwer diagram.
"""

fixed_concentrations_info = """
Define defect concentrations to be kept fixed during equilibration. **Label** can be set as:
- **name**: Defect name present in defect entries (see **Data** section). The total concentration 
            of this defect species is kept fixed, but their charges are equilibrated. 
            The relative concentrations in different charge states are independent of the 
            chemical potential of the target species.  
            Example: "Sub_Fe_on_Ti".
- **element**: Element symbol. Fix the total concentration of a target element across different species. 
               The relative concentrations of defect species containing the element and in different charge states
               are equilibrated. If the element is present in more than one defect species with different elements,
               the relative concentrations will depend on chemical potentials.  
               Example: "Fe".
"""

dopant_info = """
Settings for the calculation of the doping diagram.
Charge neutrality is solved varying the concentartion of a target defect. 
The chemical potentials defined in the **Chemical Potentials** section are kept fixed. \n

Options:
- **None** : Doping diagram is not computed.
- **Donor** : A generic donor is used as variable defect species. You can set the charge and concentration range.
- **Acceptor** : A generic acceptor is used as variable defect species. You can set the charge and concentration range.
- **<element>** : If extrinsic defects are present in the defect entries, 
                you can set each extrinsic element as variable defect species. Its total concentration is assigned, 
                but the concentrations of individual defects containing the element depend on the relative 
                formation energies. 
- **custom** : Customizable dopant. You can set name, charge and concentration range. 
                There is no requirement for the defect name, if a name fits one of the naming conventions,
                the corrisponding symbol will be printed.
""" 

brouwer_diagram_info = """
Diagram of the defect concentrations as a function of the oxygen partial pressure.
Useful to compare with experiments where the oxygen partial pressure can be controlled.
"""

cache_info = """
To prevent excessive lag when changing paramenters, the calculation result is cached. 
To rerun the calculation and regenerate the plot, click **Compute**.
"""

names_info = """
Select which defect entries to display in the plot based on `name`.
"""

concentrations_mode_info = """
Select style to plot concentrations and filter display of defect entries by `name`.

Options:
- **total**: Show the sum of concentrations in all charge states for each defect species.
- **stable**: Show the concentration of the most stable charge state for each defect species.
- **all**: Show the concentrations of all charge states for all defect species.
            Filter which charge states to show by typing them in the textbox.
"""


title = """
<svg
   width="137.35327mm"
   height="26.927582mm"
   viewBox="0 0 137.35327 26.927582"
   version="1.1"
   id="svg5"
   xmlns="http://www.w3.org/2000/svg"
   xmlns:svg="http://www.w3.org/2000/svg">
  <defs
     id="defs2" />
  <g
     id="layer1"
     transform="translate(-35.33694,-51.548845)">
    <g
       aria-label="defermi"
       id="text113"
       style="font-size:37.7137px;line-height:125%;font-family:'Latin Modern Sans Quotation';-inkscape-font-specification:'Latin Modern Sans Quotation, Normal';letter-spacing:0px;word-spacing:0px;stroke-width:0.942845px">
      <path
         d="M 53.062378,76.439886 V 53.585384 c 0,-0.754274 0,-1.659402 -1.470834,-1.659402 -1.470834,0 -1.470834,0.905128 -1.470834,1.659402 v 7.54274 c -1.508548,-2.074253 -3.507374,-3.054809 -5.694769,-3.054809 -4.78964,0 -9.089001,4.337075 -9.089001,10.220412 0,5.732482 3.997652,10.182699 8.711864,10.182699 3.05481,0 4.978208,-1.961112 5.996478,-3.507374 v 1.470834 c 0,0.791988 0,1.659403 1.508548,1.659403 1.508548,0 1.508548,-0.867415 1.508548,-1.659403 z m -3.017096,-4.902781 c 0,1.169125 -0.678846,2.225109 -1.546261,3.05481 -0.791988,0.791988 -1.847972,1.508548 -3.469661,1.508548 -3.092523,0 -6.675325,-2.225108 -6.675325,-7.806736 0,-5.619341 3.884512,-7.844449 7.165603,-7.844449 1.961113,0 3.281092,1.093697 4.035366,2.225108 0.490278,0.678846 0.490278,0.791988 0.490278,1.470834 z"
         style="fill:#000080"
         id="path167" />
      <path
         d="m 76.407158,67.275457 c 0,-5.430772 -3.05481,-9.390711 -8.372441,-9.390711 -5.242204,0 -9.315284,4.676499 -9.315284,10.258126 0,5.770196 4.48793,10.333554 9.956417,10.333554 2.149681,0 4.450216,-0.641133 6.411329,-1.998826 1.131411,-0.791988 1.131411,-0.942843 1.131411,-1.244552 0,-0.754274 0,-1.923399 -0.641133,-1.923399 -0.226282,0 -0.377137,0.150855 -0.641133,0.377137 -1.961113,1.772544 -4.374789,2.413677 -6.222761,2.413677 -3.922224,0 -6.901607,-3.394233 -7.127889,-7.278744 h 13.162081 c 0.791988,0 1.659403,0 1.659403,-1.546262 z m -2.45139,-0.301709 h -12.33238 c 0.490278,-3.959939 3.281092,-6.713039 6.411329,-6.713039 5.581627,0 5.845623,5.468487 5.921051,6.713039 z"
         style="fill:#800000"
         id="path169" />
      <path
         d="m 94.170303,53.509957 v -0.527992 c 0,-0.980556 -0.03771,-1.01827 -1.508548,-1.244552 -1.169125,-0.188568 -2.262822,-0.188568 -2.564532,-0.188568 -3.582801,0 -7.165603,1.659402 -7.165603,5.129063 v 1.772544 h -2.14968 c -0.452565,0 -1.659403,0 -1.659403,1.169124 0,1.206839 1.169124,1.206839 1.659403,1.206839 h 2.14968 v 15.613471 c 0,0.754274 0,1.659403 1.470835,1.659403 1.470834,0 1.470834,-0.905129 1.470834,-1.659403 V 60.826415 h 3.809084 c 0.452564,0 1.659402,0 1.659402,-1.169125 0,-1.206838 -1.169124,-1.206838 -1.659402,-1.206838 h -3.884511 v -2.526818 c 0,-0.791988 0,-1.131411 1.43312,-1.621689 0.905129,-0.30171 1.810258,-0.377137 2.7531,-0.377137 0.754274,0 1.772544,0.03771 3.130237,0.490278 0.339424,0.113141 0.414851,0.150855 0.527992,0.150855 0.527992,0 0.527992,-0.490279 0.527992,-1.055984 z"
         style="fill:#800000"
         id="path171" />
      <path
         d="m 111.7826,67.275457 c 0,-5.430772 -3.05481,-9.390711 -8.37244,-9.390711 -5.242207,0 -9.315286,4.676499 -9.315286,10.258126 0,5.770196 4.48793,10.333554 9.956416,10.333554 2.14968,0 4.45022,-0.641133 6.41133,-1.998826 1.13141,-0.791988 1.13141,-0.942843 1.13141,-1.244552 0,-0.754274 0,-1.923399 -0.64113,-1.923399 -0.22628,0 -0.37714,0.150855 -0.64114,0.377137 -1.96111,1.772544 -4.37478,2.413677 -6.22276,2.413677 -3.92222,0 -6.901603,-3.394233 -7.127885,-7.278744 H 110.1232 c 0.79198,0 1.6594,0 1.6594,-1.546262 z m -2.45139,-0.301709 H 96.998829 c 0.490278,-3.959939 3.281091,-6.713039 6.411331,-6.713039 5.58162,0 5.84562,5.468487 5.92105,6.713039 z"
         style="fill:#000080"
         id="path173" />
      <path
         d="m 128.26347,59.65729 v -0.527992 c 0,-0.791988 0,-1.055983 -0.79199,-1.055983 -0.98055,0 -5.20449,0.339423 -7.61816,5.393059 h -0.0377 v -3.35652 c 0,-0.678846 0,-1.659402 -1.3954,-1.659402 -1.43312,0 -1.43312,0.942842 -1.43312,1.659402 v 16.330032 c 0,0.754274 0,1.659403 1.47083,1.659403 1.47083,0 1.47083,-0.905129 1.47083,-1.659403 v -8.108445 c 0,-4.563358 3.58281,-7.429599 7.46732,-7.618167 0.75427,-0.03771 0.86741,-0.03771 0.86741,-1.055984 z"
         style="fill:#000080"
         id="path175" />
      <path
         d="M 161.86637,76.439886 V 64.484643 c 0,-4.299361 -2.18739,-6.411328 -6.41133,-6.411328 -3.73365,0 -5.88333,2.45139 -6.86389,4.450216 -0.64113,-3.130237 -2.86624,-4.450216 -6.22276,-4.450216 -3.05481,0 -5.35534,1.621689 -6.75075,4.223934 h -0.0377 v -2.187395 c 0,-0.71656 0,-1.659402 -1.43313,-1.659402 -1.47083,0 -1.47083,0.905128 -1.47083,1.659402 v 16.330032 c 0,0.791988 0,1.659403 1.50855,1.659403 1.50855,0 1.50855,-0.867415 1.50855,-1.659403 v -9.767848 c 0,-3.545088 2.30053,-6.22276 5.5062,-6.22276 4.07308,0 4.56335,2.187394 4.56335,4.299361 v 11.691247 c 0,0.791988 0,1.659403 1.50855,1.659403 1.50855,0 1.50855,-0.867415 1.50855,-1.659403 v -9.767848 c 0,-3.545088 2.30054,-6.22276 5.5062,-6.22276 4.07308,0 4.56336,2.187394 4.56336,4.299361 v 11.691247 c 0,0.791988 0,1.659403 1.50855,1.659403 1.50854,0 1.50854,-0.867415 1.50854,-1.659403 z"
         style="fill:#000080"
         id="path177" />
      <path
         d="M 172.27536,76.439886 V 60.109854 c 0,-0.754274 0,-1.659402 -1.47083,-1.659402 -1.47084,0 -1.47084,0.905128 -1.47084,1.659402 v 16.330032 c 0,0.754274 0,1.659403 1.47084,1.659403 1.47083,0 1.47083,-0.905129 1.47083,-1.659403 z m 0.41485,-22.741361 c 0,-1.621689 -0.86741,-1.621689 -1.6594,-1.621689 h -0.37714 c -0.79199,0 -1.6594,0 -1.6594,1.621689 0,1.508548 0.64113,1.659403 1.69712,1.659403 h 0.30171 c 1.05598,0 1.69711,-0.150855 1.69711,-1.659403 z"
         style="fill:#000080"
         id="path179" />
    </g>
  </g>
</svg>
"""



import streamlit as st
import seaborn as sns
import matplotlib
import pandas as pd

from defermi import DefectsAnalysis



def get_default_df():
    dtypes = {
        "Include": "boolean",
        "name": "string",
        "charge": "Int64",
        "multiplicity": "Int64",
        "energy_diff": "float64",
        "bulk_volume": "Int64"
    }
    df = pd.DataFrame({col: pd.Series(dtype=dt) for col, dt in dtypes.items()})
    return df


def set_defaults():
    sns.set_theme(context='talk',style='whitegrid')

    st.session_state['fontsize'] = 16
    st.session_state['label_size'] = 16
    st.session_state['npoints'] = 80
    #st.session_state['pressure_range'] = (1e-35,1e30)
    st.session_state['figsize'] = (8, 8)
    st.session_state['alpha'] = 0.0
    st.session_state['max_temperature'] = 2000

    if "color_sequence" not in st.session_state:
        st.session_state['color_sequence'] = matplotlib.color_sequences['tab10']
        st.session_state['color_sequence'] += matplotlib.color_sequences['tab20']
        st.session_state['color_sequence'] += matplotlib.color_sequences['Pastel1']

    if st.session_state.da:
        df = st.session_state['complete_dataframe'].dropna(
                subset=['name','charge','multiplicity','energy_diff','bulk_volume'])
        full_da = DefectsAnalysis.from_dataframe(
                                        df,
                                        band_gap=st.session_state.da.band_gap,
                                        vbm=st.session_state.da.vbm)
        st.session_state['color_dict'] = {name:st.session_state['color_sequence'][idx] for idx,name in enumerate(full_da.names)}
    return



import numpy as np
import uuid
import streamlit as st

from defermi_gui.info import quenching_info, external_defects_info
from defermi_gui.utils import init_state_variable, entries_section


def thermodynamics():
    if st.session_state.da:
        init_state_variable('temperature',value=1000)
        st.markdown("### Temperature (K)")
        temperature = st.slider("Temperature (K)", min_value=0, max_value=st.session_state['max_temperature'], value=st.session_state['temperature'], 
                                step=50, key="widget_temperature",label_visibility='collapsed')
        if temperature == 0:
            temperature = 1 # prevent division by zero
        st.session_state['temperature'] = temperature

        quenching()
        external_defects()

        st.divider()


def quenching():
    """
    GUI elements to set defect quenching parameters.
    """
    init_state_variable('enable_quench',value=False)
    init_state_variable('quench_temperature',value=None)
    init_state_variable('quench_mode',value='species')
    init_state_variable('quenched_species',value=None)
    init_state_variable('quench_elements',value=None)

    enable_quench = st.checkbox("Enable quenching", value=st.session_state['enable_quench'], key="widget_enable_quench")
    st.session_state['enable_quench'] = enable_quench
    if enable_quench:
        cols = st.columns([0.45,0.45,0.1])
        with cols[2]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(quenching_info)
        with cols[0]:
            st.session_state['quench_temperature'] = 300
            quench_temperature = st.slider("Quench Temperature (K)", min_value=0, max_value=st.session_state['max_temperature'], 
                                        value=st.session_state['quench_temperature'], step=50, key="widget_quench_temperature")
            st.session_state['quench_temperature'] = quench_temperature
        if st.session_state['quench_temperature'] == 0:
            st.session_state['quench_temperature'] = 1 

        with cols[1]:
            index = 0 if st.session_state['quench_mode'] == 'species' else 1
            quench_mode = st.radio("Quenching mode",("species","elements"),horizontal=True,key="widget_quench_mode",index=index)

        if quench_mode == "species":
            species = set()
            for entry in st.session_state.da:
                for df in entry.defect:
                    species.add(df.name)
            default = st.session_state['quenched_species'] or species
            if st.session_state['quenched_species']:
                for name in st.session_state['quenched_species']:
                    if name not in species:
                        default = species
            quenched_species = st.multiselect("Select quenched species",options=species,default=default,key='widget_quenched_species')
            quench_elements = False

        elif quench_mode == "elements":
            species = set()
            for entry in st.session_state['da']:
                if entry.defect.type == 'Vacancy':
                    species.add(entry.defect.name)
                else:
                    for df in entry.defect:
                        species.add(df.specie)
            if st.session_state['quench_elements']:
                default = st.session_state['quenched_species'] or species
            else:
                default = species
            quenched_species = st.multiselect("Select quenched elements",options=species,default=default,key='widget_quenched_elements')
            quench_elements = True
    
        st.session_state['quenched_species'] = quenched_species
        st.session_state['quench_elements'] = quench_elements
    else:
        st.session_state['quenched_species'] = None
        st.session_state['quench_elements'] = False
        st.session_state['quench_temperature'] = None



def external_defects():
    """
    GUI elements to set external defects.
    """
    cols = st.columns([0.9,0.1])
    with cols[0]:
        st.markdown("**External defects**")
    with cols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(external_defects_info)

    conc_label = r"log₁₀(c (cm⁻³))"
    labels_types_dict = {
                    'Name':str,
                    'Charge':int,
                    conc_label:int}
    entries = entries_section(
                            widget_key='external_defects',
                            labels_types_dict=labels_types_dict,
                            columns=[0.11, 0.26, 0.26, 0.26, 0.11])
    
    st.session_state['external_defects'] = [{
                    'name':e['Name'],
                    'charge':e['Charge'],
                    'conc':float(10 ** e[conc_label])
                    } for e in entries if e['Name']]
        



import os
import json
import streamlit as st
from monty.json import MontyDecoder

from defermi_gui.info import dos_info
from defermi_gui.utils import init_state_variable

def dos():
    """
    Import DOS file or set effective mass
    """
    if st.session_state.da:
        st.markdown("# Thermodynamic Parameters")
        cols = st.columns([0.9,0.1])
        with cols[0]:
            st.markdown("**Density of states**")
        with cols[1]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(dos_info)

        init_state_variable('dos',value=None)

        cols = st.columns([0.2, 0.8])
        with cols[0]:
            if not st.session_state['dos']:
                index = 0
            elif st.session_state['dos_type'] == "$m^*/m_e$":
                index = 0
            elif st.session_state['dos_type'] == 'DOS':
                index = 1
            dos_type = st.radio("Select",("$m^*/m_e$","DOS"),horizontal=True,index=index,label_visibility='collapsed',key='widget_dos_type')
            st.session_state['dos_type'] = dos_type
        with cols[1]:
            if dos_type == "DOS":
                subcols = st.columns([0.5,0.5])
                with subcols[0]:
                    with st.expander('🗄️ Database'):
                        subsubcols = st.columns([0.7,0.3])
                        with subsubcols[0]:
                            composition = st.text_input(label='Composition',key='widget_dos_composition_DB')
                            if composition:
                                if st.button("Pull",key='widget_pull_dos_DB'):
                                    dos = pull_dos_from_stable_composition(composition=composition)
                                    st.session_state['dos'] = dos
                with subcols[1]:
                    uploaded_dos = st.file_uploader("Upload", type=["json"], label_visibility="collapsed")
                    if uploaded_dos is not None:
                        data_bytes = uploaded_dos.getvalue() # file content as bytes
                        json_str = data_bytes.decode('utf-8') # decode bytes to string (JSON text)                      
                        dos = MontyDecoder().decode(json_str)
                        st.session_state['dos'] = dos
            elif dos_type == '$m^*/m_e$':
                cols = st.columns(2)
                with cols[0]:
                    if st.session_state['dos'] and type(st.session_state['dos']) == dict and 'm_eff_e' in st.session_state['dos']:
                        value = st.session_state['dos']['m_eff_e']
                    else:
                        st.session_state['dos'] = {}
                        value = 1.0
                    m_eff_e = st.number_input(f"e", value=value,step=0.1, key='widget_dos_m_eff_e')
                    st.session_state['dos']['m_eff_e'] = m_eff_e
                with cols[1]:
                    if st.session_state['dos'] and type(st.session_state['dos']) == dict and 'm_eff_h' in st.session_state['dos']:
                        value = st.session_state['dos']['m_eff_h']
                    else:
                        value = 1.0
                    m_eff_h = st.number_input(f"h", value=value, step=0.1, key='widget_dos_m_eff_h')
                    st.session_state['dos']['m_eff_h'] = m_eff_h

        st.divider()


def pull_dos_from_stable_composition(composition,thermo_types=['GGA_GGA+U'],**kwargs):
    import base64
    from defermi.tools.materials_project import MPDatabase

    API_KEY = base64.b64decode('Q0FVMk8yODZmRUI2cGJWOUszTU9qblFFUFJkZW9BQXg=').decode()
    dos = MPDatabase(API_KEY=API_KEY).get_dos_from_stable_composition(
                                                                        composition=composition,
                                                                        thermo_types=thermo_types,
                                                                        **kwargs)
    return dos




import streamlit as st

from defermi import DefectsAnalysis
from defermi_gui.defaults import get_default_df
from defermi_gui.utils import init_state_variable


def initialize_state_variables():
    init_state_variable('session_loaded',value=False)
    init_state_variable('session_name',value='session')
    init_state_variable('band_gap',value=None)


def initialize_defects_analysis(df_complete):
    st.session_state.da = None # initialize da
    df = df_complete.dropna(
            subset=['name','charge','multiplicity','energy_diff','bulk_volume'])
    if df.empty:
        st.warning('Dataset is empty')
    elif st.session_state['band_gap'] and 'vbm' in st.session_state:
        df_to_import = df[df["Include"] == True] # keep only selected rows
        st.session_state.da = DefectsAnalysis.from_dataframe(
                                                    df_to_import,
                                                    band_gap=st.session_state['band_gap'],
                                                    vbm=st.session_state['vbm'],
                                                    include_data=False)
    return


def initialize_complete_dataframe():
    key = 'complete_dataframe'
    input_key = 'input_dataframe'  
    if input_key in st.session_state:
        if key not in st.session_state or st.session_state[key].empty:
            st.session_state[key] = st.session_state[input_key]
    else:
        if key not in st.session_state:
            st.session_state[key] = get_default_df()
    return




import numpy as np
import uuid
import streamlit as st
import matplotlib.pyplot as plt

from pymatgen.core.composition import Composition

from defermi.defects import get_defect_from_string
from defermi.plotter import plot_pO2_vs_concentrations, plot_pO2_vs_fermi_level
from defermi_gui.info import *
from defermi_gui.utils import init_state_variable, download_plot, _get_axis_limits_with_widgets, _filter_concentrations, entries_section


def oxygen_ref():
    init_state_variable('oxygen_ref',value=-4.95)
    init_state_variable('pressure_range',value=None)
    subcols = st.columns([0.2,0.1,0.4,0.3])
    with subcols[0]:
        oxygen_ref = st.number_input('**μO (0K, p0) [eV]**', value=st.session_state['oxygen_ref'], step=0.5, key='widget_oxygen_ref',label_visibility='visible')
        st.session_state['oxygen_ref'] = oxygen_ref
    with subcols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(oxygen_ref_info)
    with subcols[2]:
        def update_pressure_range():
            min_conc, max_conc = st.session_state['widget_pressure_range']
            st.session_state['pressure_range'] = ( float(10**min_conc), float(10**max_conc) )
            return
        if st.session_state['pressure_range']:
            value = int(np.log10(float(st.session_state['pressure_range'] [0]))), int(np.log10(float(st.session_state['pressure_range'] [1])))
        else:
            value = (-35,30) 
            st.session_state['pressure_range'] = (1e-35,1e30)       
        st.slider(r"pO₂ range: log₁₀(p (atm))",min_value=-35,max_value=30,value=value,step=1, 
                                            key="widget_pressure_range",on_change=update_pressure_range)

def precursors():
    if st.session_state.da:
        da = st.session_state.da
        cols = st.columns([0.9,0.1])
        with cols[0]:
            st.markdown("##### Precursors")
        with cols[1]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(precursors_info)

        init_state_variable('precursor_entries',value=[]) 
        
        cols = st.columns([0.1, 0.25, 0.25, 0.3, 0.1])
        with cols[0]:
            add_precursors = st.button("➕",key="widget_add_precursor")
            if add_precursors:
                # Generate a unique ID for this entry
                entry_id = str(uuid.uuid4())
                st.session_state.precursor_entries.append({
                    "id": entry_id,
                    "composition": "",
                    "energy": 0.0
                })

        def remove_precursor_entry(entry_id):
            for idx,entry in enumerate(st.session_state['precursor_entries']):
                if entry['id'] == entry_id:
                    del st.session_state['precursor_entries'][idx]

        init_state_variable('precursor_DB_warning',value=None)
        def set_precursor_energy_from_DB(entry):
            if entry['composition']:
                energy_pfu = pull_stable_energy_pfu_from_composition(composition=entry['composition'])
                st.session_state[f"widget_energy_{entry['id']}"] = energy_pfu
            else:
                st.session_state['precursor_DB_warning'] = 'Enter composition to pull energy from MP database'
            return 


        for entry in st.session_state['precursor_entries']:
            with cols[1]:
                entry["composition"] = st.text_input("Composition", value=entry["composition"], key=f"widget_comp_{entry['id']}")
            with cols[2]:
                widget_key = f"widget_energy_{entry['id']}"
                if widget_key not in st.session_state:
                    st.session_state[widget_key] = entry['energy']
                energy = st.number_input("Energy p.f.u (eV)", step=1.0, key=f"widget_energy_{entry['id']}")
                entry["energy"] = energy
            with cols[3]:
                st.write('')
                st.button('🗄️ Materials Project DB',on_click=set_precursor_energy_from_DB,args=[entry],key=f"widget_pull_{entry['id']}")
            if st.session_state['precursor_DB_warning']:
                st.error(st.session_state['precursor_DB_warning'])
                st.session_state['precursor_DB_warning'] = None
            with cols[4]:
                st.write('')
                st.button("🗑️", on_click=remove_precursor_entry, args=[entry['id']], key=f"widget_del_{entry['id']}")

        if st.session_state.da.elements == ['O']:
            st.session_state['precursors'] = None
        else:
            st.session_state['precursors'] = {
                            entry["composition"]: entry["energy"] 
                            for entry in st.session_state.precursor_entries
                            if entry["composition"]}
            

        cols = st.columns([0.9,0.1])
        with cols[0]:
            st.markdown("##### Fixed concentrations")
        with cols[1]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(fixed_concentrations_info)

        conc_label = 'log₁₀(concentration (cm⁻³))'
        labels_types_dict = {'Label':str,conc_label:int}
        entries = entries_section(
                                widget_key='fixed_concentrations',
                                labels_types_dict=labels_types_dict,
                                columns = [0.1,0.25,0.25,0.4])
        
        fixed_conc = {entry["Label"]:float(10**entry[conc_label]) for entry in entries}
        #fixed_conc = {k:float(10**v) for k,v in fixed_conc_log.items()}

        st.session_state['fixed_concentrations'] = fixed_conc



def filter_entries_with_missing_elements():
    """
    Remove defect entries with elements missing from precursors from brouwer diagram dataset.
    """
    if "precursors" in st.session_state and st.session_state.da:
        precursors = st.session_state['precursors']
        da = st.session_state.da
        # remove defect entries with missing precursors from brouwer diagram dataset
        elements_in_precursors = set()
        if precursors:
            for comp in precursors:
                if comp:
                    for element in Composition(comp).elements:
                        elements_in_precursors.add(element.symbol)


        filter_elements = set('O')
        missing_elements = set()
        for label in st.session_state['fixed_concentrations']: # fixed elements dont need precursors
            try:
                defect = get_defect_from_string(label)
                el = defect.specie
            except:
                el = label
            filter_elements.add(el)

        for el in da.elements:
            if el in elements_in_precursors:
                filter_elements.add(el)
            elif el not in filter_elements and el != 'O': # "O" already in oxygen_ref
                missing_elements.add(el)


        cols = st.columns(5)
        for idx,el in enumerate(missing_elements):
            ncolumns = 5
            col_idx = idx%ncolumns
            with cols[col_idx]:
                st.warning(f'{el} missing from precursors')

        if filter_elements:
            try:
                brouwer_da = da.filter_entries(elements=filter_elements)
            except AttributeError:
                st.warning('No entries for Brouwer diagram calculation')
                brouwer_da = None
        else:
            brouwer_da = None
        
        st.session_state['brouwer_da'] = brouwer_da
        st.session_state['quenched_species_brouwer'] = []
        if brouwer_da:
            if st.session_state['quenched_species']:
                if st.session_state['quench_elements']:
                    for entry in brouwer_da.entries:
                        for df in entry.defect:
                            if df.type == 'Vacancy':
                                if df.name in st.session_state['quenched_species']:
                                    st.session_state['quenched_species_brouwer'].append(df.name)
                            else:
                                if df.specie in st.session_state['quenched_species']:
                                    st.session_state['quenched_species_brouwer'].append(df.specie)
                else:
                    for species in st.session_state['quenched_species']:
                        if species in brouwer_da.names:
                            st.session_state['quenched_species_brouwer'].append(species)
    else:
        st.session_state.pop('brouwer_diagram_figure',None)
        st.session_state.pop('fermi_level_brouwer_figure',None)
        



def pull_stable_energy_pfu_from_composition(composition,thermo_types=['GGA_GGA+U'],**kwargs):
    import base64
    from defermi.tools.materials_project import MPDatabase

    API_KEY = base64.b64decode('Q0FVMk8yODZmRUI2cGJWOUszTU9qblFFUFJkZW9BQXg=').decode()
    energy_pfu = MPDatabase(API_KEY=API_KEY).get_stable_energy_pfu_from_composition(
                                                                                composition=composition,
                                                                                thermo_types=thermo_types,
                                                                                **kwargs)
    return energy_pfu


@st.cache_data
def compute_brouwer_diagram(_brouwer_da):
    _brouwer_da.plot_brouwer_diagram(
                            bulk_dos=st.session_state['dos'],
                            temperature=st.session_state['temperature'],
                            quench_temperature=st.session_state['quench_temperature'],
                            quenched_species=st.session_state['quenched_species_brouwer'],
                            quench_elements = st.session_state['quench_elements'],
                            fixed_concentrations = st.session_state['fixed_concentrations'],
                            precursors=st.session_state['precursors'],
                            oxygen_ref=st.session_state['oxygen_ref'],
                            pressure_range=st.session_state['pressure_range'],
                            external_defects=st.session_state['external_defects'],
                            npoints=st.session_state['npoints']
                        )
    return _brouwer_da.thermodata


def get_pO2_vs_fermi_level_figure(xlim,ylim=None):
    if st.session_state['brouwer_thermodata']:    
        figsize = (6,6)
        da = st.session_state.da
        thermodata = st.session_state.brouwer_thermodata

        fig = plot_pO2_vs_fermi_level(
                partial_pressures=thermodata.partial_pressures,
                fermi_levels=thermodata.fermi_levels,
                band_gap=da.band_gap,
                figsize=figsize,
                fontsize=st.session_state['fontsize'],
                xlim=xlim,
                ylim=ylim
        )
        fig.grid()
        fig.title('Brouwer diagram')
        fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
        fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
        ax = fig.gca()
        fig = ax.get_figure()
        fig.patch.set_alpha(st.session_state['alpha'])
        ax.patch.set_alpha(st.session_state['alpha'])
        return fig



def main():

    st.set_page_config(layout="wide")
    cols = st.columns([0.4,0.6])
    with cols[0]:
        st.title('Brouwer Diagram')
    with cols[1]:
        with st.popover(label='ℹ️',help='Info',type='tertiary'):
            st.write(brouwer_diagram_info)
    if st.session_state.da and 'O' in st.session_state.da.elements:
        st.write('')
        oxygen_ref()
        precursors()
        filter_entries_with_missing_elements()

        cols = st.columns([0.3,0.2,0.2,0.3])
        with cols[1]:
            if st.button('Compute',key='widget_clear_cache_brouwer'):
                compute_brouwer_diagram.clear()
        with cols[2]:
            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                st.write(cache_info)

        st.divider()


        if "dos" in st.session_state and "precursors" in st.session_state:
            if st.session_state['precursors'] or st.session_state.brouwer_da.elements==['O']:
                pressure_range = st.session_state['pressure_range']
                brouwer_da = st.session_state['brouwer_da']
                if brouwer_da:
                    cols = st.columns([0.7,0.3])
                    with cols[1]:
                        default_xlim = int(np.log10(pressure_range[0])) , int(np.log10(pressure_range[1]))
                        set_xlim, xlim = _get_axis_limits_with_widgets(
                                                                    label='xlim (log)',
                                                                    key='brouwer',
                                                                    default=default_xlim,
                                                                    boundaries=default_xlim) 
                        xlim = (float(10**xlim[0]) , float(10**xlim[1]))
                        xlim = xlim if set_xlim else pressure_range

                        set_ylim, ylim = _get_axis_limits_with_widgets(
                                                                    label='ylim (log)',
                                                                    key='brouwer',
                                                                    default=(-20,25),
                                                                    boundaries=(-50,30))
                        ylim = (float(10**ylim[0]) , float(10**ylim[1]))
                        ylim = ylim if set_ylim else None   

                        try:
                            brouwer_thermodata = compute_brouwer_diagram(_brouwer_da=brouwer_da) # leading underscore tells streamlit not to hash the argument
                        except ValueError as e:
                            if "Self-consistent Fermi level solver failed" in str(e):
                                brouwer_thermodata = None
                        if brouwer_thermodata:
                            dc = brouwer_thermodata.defect_concentrations[0]
                            output, names, charges, colors = _filter_concentrations(dc,key='brouwer')

                    if brouwer_thermodata:
                        with cols[0]:  
                            fig = plot_pO2_vs_concentrations(
                                                        thermodata=brouwer_thermodata,
                                                        output=output,
                                                        figsize=st.session_state['figsize'],
                                                        fontsize=st.session_state['fontsize'],
                                                        xlim=xlim,
                                                        ylim=ylim,
                                                        colors=colors,
                                                        names=names,
                                                        charges=charges)                                           

                            fig.grid()
                            fig.xlabel(plt.gca().get_xlabel(), fontsize=st.session_state['label_size'])
                            fig.ylabel(plt.gca().get_ylabel(), fontsize=st.session_state['label_size'])
                            ax = fig.gca()
                            fig = ax.get_figure()
                            fig.patch.set_alpha(st.session_state['alpha'])
                            ax.patch.set_alpha(st.session_state['alpha'])
                            st.session_state['brouwer_thermodata'] = brouwer_thermodata
                            st.session_state['brouwer_diagram_figure'] = fig
                            st.pyplot(fig, clear_figure=False, width="stretch")

                            fig_fermi = get_pO2_vs_fermi_level_figure(xlim)
                            st.session_state['fermi_level_brouwer_figure'] = fig_fermi

                        with cols[1]:
                            with st.popover(label='ℹ️',help='Info',type='tertiary'):
                                st.write(concentrations_mode_info)
                            st.write('')
                            download_plot(fig=fig,filename='brouwer_diagram.pdf')
                    else:
                        st.session_state.pop('fermi_level_brouwer_figure',None)
                        st.warning("Self-consistent Fermi level solver failed: "
                                    "one or more defect concentrations likely diverge.")
                        st.warning("Try to adjust the oxygen partial pressure range.")
                        

    elif st.session_state.da and 'O' not in st.session_state.da.elements:
        st.warning('Brouwer analysis concerns only systems containing Oxygen')


if __name__ == '__main__':
    main()


import streamlit as st
import matplotlib.pyplot as plt

from defermi_gui.brouwer import get_pO2_vs_fermi_level_figure
from defermi_gui.ctls import CTLsPlotter
from defermi_gui.doping import get_doping_vs_fermi_level_figure
from defermi_gui.formation_energies import FormationEnergiesPlotter
from defermi_gui.info import title
from defermi_gui.utils import svg_logo, init_state_variable, insert_space

FIGSIZE = (7,7)
WIDTH = "content"

cols = st.columns(2)
with cols[0]:
    fig = None
    if 'formation_energies_figure' in st.session_state:
        fig = st.session_state['formation_energies_figure']
    elif st.session_state.da:
        plotter = FormationEnergiesPlotter(st.session_state.da) 
        entries,colors = plotter.get_entries_and_colors(filter_names=False)
        fig = plotter.get_figure(entries,colors)
    if fig:
        fig.gca().set_title('')
        fig.set_size_inches(*FIGSIZE)
        fig.tight_layout()
        subcols = st.columns([0.32,0.68])
        with subcols[1]:
            st.markdown('#### Formation Energies')
        st.pyplot(fig, clear_figure=False, width=WIDTH)

with cols[1]:
    fig = None
    if 'ctls_figure' in st.session_state:
        fig = st.session_state['ctls_figure']
    elif st.session_state.da:
        plotter = CTLsPlotter(st.session_state.da) 
        entries = plotter.get_entries(filter_names=False)
        fig = plotter.get_figure(entries)
    if fig:
        fig.gca().set_title('')
        fig.set_size_inches(*FIGSIZE)
        fig.tight_layout()
        subcols = st.columns([0.25,0.75])
        with subcols[1]:
            st.markdown('#### Charge Transition Levels')
        st.pyplot(fig, clear_figure=False, width=WIDTH)


with cols[0]:
    st.write('')
    if 'doping_diagram_figure' in st.session_state:
        fig = st.session_state['doping_diagram_figure']
        fig.set_size_inches(*FIGSIZE)
        fig.tight_layout()
        fig.gca().set_title('')
        subcols = st.columns([0.37,0.63])
        with subcols[1]:
            st.markdown('#### Doping Diagram')
        st.pyplot(fig, clear_figure=False, width=WIDTH)

    

with cols[1]:
    st.write('')
    if 'brouwer_diagram_figure' in st.session_state:
        fig = st.session_state['brouwer_diagram_figure']
        fig.set_size_inches(*FIGSIZE)
        fig.tight_layout()
        fig.gca().set_title('')
        subcols = st.columns([0.35,0.65])
        with subcols[1]:
            st.markdown('#### Brouwer Diagram')
        st.pyplot(fig, clear_figure=False, width=WIDTH)






